// kkcluster.cpp : Defines the entry point for the console application.
//
#ifndef H_CLUSTER
#define H_CLUSTER 1
#define DEBUG 1

#include "kkcluster.h"
#include <limits>

namespace Clustering{

	//
	// compute all the centroids
	//
	void computeCentroid(const Space & sp, const Assignement & a, Points & centroids) 
	{
		// there are k clusters
		unsigned int k = centroids.size();
		std::vector<unsigned int> numPointsInCluster(k, 0);

		// zero the centroids
		for (unsigned int cid=0; cid<k; cid++){  // for each cluster
			for (unsigned int d=0; d<sp.numDimensions(); d++){ // for each dimension
				centroids[cid][d] = 0.0;
			}
		}

		IDPoint pid = 0;
		// update the centroids
		BOOST_FOREACH(Assignement::value_type cid, a){ // for each id in cluster

			std::cout << "Point " << pid << " Assigned to cid=" << cid.first << std::endl;
			for (unsigned int d=0; d<sp.numDimensions(); d++){
				centroids[cid.first][d]+=sp.point(pid)[d];   // increment this dimension
			}
			numPointsInCluster[cid.first]++;    // another point seen for this cluster;
			pid++;
		}
		for (unsigned int cid=0; cid<k; cid++){  // for each cluster
			for (unsigned int d=0; d<sp.numDimensions(); d++){ // for each dimension
				centroids[cid][d]/= (1.0 * (numPointsInCluster[cid]));
			}
		}
	};

	//
	// compute kmeans
	//
	void kmeans(const Space & sp, unsigned int k, Assignement & A){

		// random assigment for this point to K cluster
		for (unsigned int i=0; i<sp.numPoints(); i++) 
			A[i]= std::make_pair((i % k), std::numeric_limits<double>::max( ));

		// build k centroids
		Points centroids;
		for (unsigned int i=0; i<k; i++){	
			Point centroid(sp.numDimensions(), 0);
			centroids.push_back(centroid);
		}
		computeCentroid(sp, A, centroids);

		bool move = true;
		while (move){          // there moving points from one cluster to cluster

			move = false;
			// for each point in the space
			Distance min, d;
			IDCluster cid;
			IDPoint pid = 0;

			std::cout << "Assignment " << A;
			std::cout << "Centroids: " << centroids;

			BOOST_FOREACH(Points::value_type p, sp.points()){  // foreach point

				min = A[pid].second;                          // get the distance from current centroid
				cid = 0;
				// for each centroid in centroids 
				BOOST_FOREACH(Points::value_type c, centroids){  // foreach centroid

					d = dist(p, c);     // compute the dist
					if (d < min){       // improve?  
						move = true;    // new move   
						min = d;        // update the min    
						A[pid] = std::make_pair(cid, d);   // assign this point pid to cluster cid and store the distance
#ifdef DEBUG2
						std::cout << "Point move: " << p;	
						std::cout << " into cluster " << cid << " distance " << d << std::endl;
#endif
					}
					cid++;              // another centroid seen
				} // foreach centroid
				pid++;                  // another point seen
			} // foreach point

			// recompute centroids;
			computeCentroid(sp, A, centroids);
			
	int a;
	std::cin >> a;
		} // move
	}; // k-means
}

#endif